/*** Scalar Functions Performance ***/
IF EXISTS(SELECT 1 FROM sys.objects WHERE name='fngetSalesTotalByProductId' AND type='FN') -- Scalar function
DROP FUNCTION dbo.fngetSalesTotalByProductId
GO

CREATE FUNCTION dbo.fngetSalesTotalByProductId
(
@ProductId bigint
)
RETURNS money
AS
BEGIN
		DECLARE @Total bigint
		
		SELECT @Total = SUM(LineTotal)
		FROM Sales.SalesOrderDetail  
		WHERE ProductID = @ProductId 
		
	RETURN(@Total)
END

GO

/** Example using Scalar functions ***/
SELECT P.ProductID,dbo.fngetSalesTotalByProductId(P.ProductID) AS SumTotal 
FROM Production.product P 
WHERE ProductNumber LIKE 'fr%' AND dbo.fngetSalesTotalByProductId(P.ProductID) IS NOT NULL

GO

/*** Inline Function Performance ***/
IF EXISTS(SELECT 1 FROM sys.objects WHERE name='fngetSalesTotalByProductId_InlineTVF' AND type='IF') -- Scalar function
DROP FUNCTION dbo.fngetSalesTotalByProductId_InlineTVF
GO

CREATE FUNCTION dbo.fngetSalesTotalByProductId_InlineTVF
(
@ProductId bigint
)
RETURNS TABLE
AS
RETURN
(
			SELECT SUM(LineTotal) AS Sumtotal
			FROM Sales.SalesOrderDetail 
			WHERE ProductID = @ProductId 
)

GO

/***Example using Inline function ***/
SELECT P.ProductID,(SELECT Sumtotal FROM dbo.fngetSalesTotalByProductId_inlinetvf(P.ProductID)) AS SumTotal 
FROM Production.product P 
WHERE ProductNumber LIKE 'fr%' AND (SELECT Sumtotal FROM dbo.fngetSalesTotalByProductId_inlinetvf(P.ProductID)) IS NOT NULL

GO

IF EXISTS(SELECT 1 FROM sys.objects WHERE name='fngetSalesTotalbyProductId_MultistatementTVF' AND type='TF') -- Scalar function
DROP FUNCTION dbo.fngetSalesTotalbyProductId_MultistatementTVF
GO

CREATE FUNCTION dbo.fngetSalesTotalbyProductId_MultistatementTVF
(
@ProductId bigint
)
RETURNS @Sumtotal TABLE
(
SumTotal bigint
) 
AS
BEGIN		
		
		INSERT INTO @Sumtotal
		SELECT SUM(LineTotal) AS Sumtotal
		FROM Sales.SalesOrderDetail sd 
		WHERE ProductID = @ProductId 
		
		RETURN 	
END 

GO

/***Example using Multistatement function ***/
SELECT P.ProductID,(SELECT Sumtotal FROM dbo.fngetSalesTotalByProductId_MultistatementTVF(P.ProductID)) AS SumTotal 
FROM Production.product P 
WHERE ProductNumber LIKE 'fr%' AND (SELECT Sumtotal FROM dbo.fngetSalesTotalByProductId_MultistatementTVF(P.ProductID)) IS NOT NULL

GO